package com.lilosoft.core.queue.service;

import com.lilosoft.core.queue.command.Command;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.LinkedBlockingQueue;

/** 
 * @ClassName: FileTransQueueService 
 * @Description: 文件转换服务
 * @author: YanJie
 */
public class FileTransQueueService extends Thread {
	private boolean status = true;
	protected static FileTransQueueService instance = new FileTransQueueService();
	protected Logger logger = LoggerFactory.getLogger(getClass());

	private final LinkedBlockingQueue<Command> queue = new LinkedBlockingQueue<Command>(100);

	public synchronized static FileTransQueueService getInstance() {
		if (instance == null) {
			instance = new FileTransQueueService();
		}
		return instance;
	}
	
	@Override
	public void run() {
		while(status){
			execute();
		}
	}
	
	/**
	 * 向队列推送消息
	 * @param command
	 */
	public void putCommand(Command command) {
		try{
			/**
			 * LinkedBlockingQueue.put()：首选。队满是阻塞
			 * LinkedBlockingQueue.offer()：队满时返回false
			 */
			queue.put(command);
//			if (queue.isProcessingCompleted()) {
//				queue.setProcessingCompleted(false);
//			}
			System.out.println(queue.size());
		} catch (Exception e) {	
			e.printStackTrace();
		}
	}

	/**
	 * 获取队列未完成数量
	 * @return
	 */
	public int getQueueSize(){
		return queue.size();
	}
	
	private void execute(){
		try {
			/**
			 * LinkedBlockingQueue.take()：首选。当队列为空时阻塞
			 * LinkedBlockingQueue.poll()：弹出队顶元素，队列为空时，返回空
			 */
			Command command = queue.take();
			if (command != null) {
				command.execute();
			}
		} catch (Exception e) {	
			e.printStackTrace();
		}
	}
	
	public void close(){
		status = false;
	}
}
